from typing import List, Optional

from pydantic import BaseModel, EmailStr, Field, field_validator

# Import at module level to avoid circular imports
from app.schemas.client import Client


# Shared properties
class UserBase(BaseModel):
    email: EmailStr
    full_name: str
    company_name: Optional[str] = None
    address: Optional[str] = None
    phone: Optional[str] = None
    is_active: bool = True


# Properties to receive via API on creation
class UserCreate(UserBase):
    password: str = Field(..., min_length=8)
    
    @field_validator("password")
    def password_validation(cls, v):
        if len(v) < 8:
            raise ValueError("Password must be at least 8 characters long")
        return v


# Properties to receive via API on update
class UserUpdate(UserBase):
    email: Optional[EmailStr] = None
    full_name: Optional[str] = None
    password: Optional[str] = None

    @field_validator("password")
    def password_validation(cls, v):
        if v is not None and len(v) < 8:
            raise ValueError("Password must be at least 8 characters long")
        return v


# Properties shared by models stored in DB
class UserInDBBase(UserBase):
    id: int

    model_config = {
        "from_attributes": True
    }


# Properties to return via API
class User(UserInDBBase):
    pass


# Properties stored in DB
class UserInDB(UserInDBBase):
    hashed_password: str


# User with clients
class UserWithClients(User):
    clients: List[Client] = []